@@ -19,11 +19,13 @@
 #include "terminalDisplay/TerminalDisplay.h"
 #include "session/SessionController.h"
 #include "session/Session.h"
+#include "session/SessionManager.h"
 
 #include "history/HistoryType.h"
 #include "history/HistoryScrollNone.h"
 #include "characters/ExtendedCharTable.h"
 #include "profile/Profile.h"
+#include "profile/ProfileManager.h"
 #include "EscapeSequenceUrlExtractor.h"
 
 using namespace Konsole;
@@ -59,8 +61,8 @@ Screen::Screen(int lines, int columns):
     _oldTotalLines(0),
     _isResize(false),
     _enableReflowLines(false),
-    _lineProperties(QVarLengthArray<LineProperty, 64>()),
-    _history(new HistoryScrollNone()),
+    _lineProperties(_lines + 1),
+    _history(std::make_unique<HistoryScrollNone>()),
     _cuX(0),
     _cuY(0),
     _currentForeground(CharacterColor()),
@@ -78,10 +80,9 @@ Screen::Screen(int lines, int columns):
     _effectiveRendition(DEFAULT_RENDITION),
     _lastPos(-1),
     _lastDrawnChar(0),
-    _escapeSequenceUrlExtractor(new EscapeSequenceUrlExtractor())
+    _escapeSequenceUrlExtractor(std::make_unique<EscapeSequenceUrlExtractor>())
 {
     _escapeSequenceUrlExtractor->setScreen(this);
-    _lineProperties.resize(_lines + 1);
     std::fill(_lineProperties.begin(), _lineProperties.end(), LINE_DEFAULT);
 
     initTabStops();
@@ -89,11 +90,7 @@ Screen::Screen(int lines, int columns):
     reset();
 }
 
-Screen::~Screen()
-{
-    delete _history;
-    delete _escapeSequenceUrlExtractor;
-}
+Screen::~Screen() = default;
 
 void Screen::cursorUp(int n)
 //=CUU
@@ -159,7 +156,7 @@ void Screen::cursorRight(int n)
     if (n > MAX_SCREEN_ARGUMENT) {
         n = MAX_SCREEN_ARGUMENT;
     }
-    _cuX = qMin(_columns - 1, _cuX + n);
+    _cuX = qMin(getScreenLineColumns(_cuY) - 1, _cuX + n);
 }
 
 void Screen::setMargins(int top, int bot)
@@ -228,7 +225,7 @@ void Screen::eraseChars(int n)
         n = MAX_SCREEN_ARGUMENT;
     }
     const int p = qBound(0, _cuX + n - 1, _columns - 1);
-    clearImage(loc(_cuX, _cuY), loc(p, _cuY), ' ');
+    clearImage(loc(_cuX, _cuY), loc(p, _cuY), ' ', false);
 }
 
 void Screen::deleteChars(int n)
@@ -241,16 +238,16 @@ void Screen::deleteChars(int n)
     }
 
     // if cursor is beyond the end of the line there is nothing to do
-    if (_cuX >= _screenLines[_cuY].count()) {
+    if (_cuX >= _screenLines.at(_cuY).count()) {
         return;
     }
 
-    if (_cuX + n > _screenLines[_cuY].count()) {
-        n = _screenLines[_cuY].count() - _cuX;
+    if (_cuX + n > _screenLines.at(_cuY).count()) {
+        n = _screenLines.at(_cuY).count() - _cuX;
     }
 
     Q_ASSERT(n >= 0);
-    Q_ASSERT(_cuX + n <= _screenLines[_cuY].count());
+    Q_ASSERT(_cuX + n <= _screenLines.at(_cuY).count());
 
     _screenLines[_cuY].remove(_cuX, n);
 
@@ -259,7 +256,7 @@ void Screen::deleteChars(int n)
                                     _effectiveBackground,
                                     _effectiveRendition, false);
 
-    for (int i = 0; i < n; i++) {
+    for (int i = 0; i < n; ++i) {
         _screenLines[_cuY].append(spaceWithCurrentAttrs);
     }
 }
@@ -270,14 +267,14 @@ void Screen::insertChars(int n)
         n = 1; // Default
     }
 
-    if (_screenLines[_cuY].size() < _cuX) {
+    if (_screenLines.at(_cuY).size() < _cuX) {
         _screenLines[_cuY].resize(_cuX);
     }
 
     _screenLines[_cuY].insert(_cuX, n, Character(' '));
 
-    if (_screenLines[_cuY].count() > _columns) {
-        _screenLines[_cuY].resize(_columns);
+    if (_screenLines.at(_cuY).count() > getScreenLineColumns(_cuY)) {
+        _screenLines[_cuY].resize(getScreenLineColumns(_cuY));
     }
 }
 
@@ -294,8 +291,9 @@ void Screen::repeatChars(int n)
     // So, a "normal" program should always use REP immediately after a visible
     // character (those other than escape sequences). So, _lastDrawnChar can be
     // safely used.
-    for (int i = 0; i < n; i++) {
+    while (n > 0) {
         displayCharacter(_lastDrawnChar);
+        --n;
     }
 }
 
@@ -363,8 +361,8 @@ void Screen::saveCursor()
 
 void Screen::restoreCursor()
 {
-    _cuX     = qMin(_savedState.cursorColumn, _columns - 1);
     _cuY     = qMin(_savedState.cursorLine, _lines - 1);
+    _cuX     = qMin(_savedState.cursorColumn, getScreenLineColumns(_cuY) - 1);
     _currentRendition   = _savedState.rendition;
     _currentForeground   = _savedState.foreground;
     _currentBackground   = _savedState.background;
@@ -385,7 +383,7 @@ bool Screen::isResize()
     return _isResize;
 }
 
-void Screen::setReflowLines(bool enable) 
+void Screen::setReflowLines(bool enable)
 {
     _enableReflowLines = enable;
 }
@@ -413,7 +411,7 @@ void toDebug(const QVector<Character> &s, bool wrapped = false)
 
 int Screen::getCursorLine()
 {
-    if (_currentModes[MODE_AppScreen] == 1) {
+    if (isAppMode()) {
         return _savedState.cursorLine;
     }
     return _cuY;
@@ -421,7 +419,7 @@ int Screen::getCursorLine()
 
 void Screen::setCursorLine(int newLine)
 {
-    if (_currentModes[MODE_AppScreen] == 1) {
+    if (isAppMode()) {
         _savedState.cursorLine = newLine;
         _cuY = qBound(0, _cuY, _lines - 1);
     } else {
@@ -445,9 +443,9 @@ void Screen::resizeImage(int new_lines, int new_columns)
     // Check if _history need to change
     if (_enableReflowLines && new_columns != _columns && _history->getLines() && _history->getMaxLines()) {
         // Join next line from _screenLine to _history
-        while (_history->isWrappedLine(_history->getLines() - 1)) {
+        while (!_screenLines.isEmpty() && _history->isWrappedLine(_history->getLines() - 1)) {
             fastAddHistLine();
-            cursorLine--;
+            --cursorLine;
         }
         auto removedLines = _history->reflowLines(new_columns);
 
@@ -465,9 +463,9 @@ void Screen::resizeImage(int new_lines, int new_columns)
             // It needs to identify the 'zsh' and calculate the new command line.
             auto sessionController = _currentTerminalDisplay->sessionController();
             auto terminal = sessionController->session()->foregroundProcessName();
-            if (terminal == QLatin1String("zsh") && cursorLine > 0 && (_lineProperties[cursorLine - 1] & LINE_WRAPPED) != 0) {
-                while (cursorLine + cursorLineCorrection > 0 && (_lineProperties[cursorLine + cursorLineCorrection - 1] & LINE_WRAPPED) != 0) {
-                    cursorLineCorrection--;
+            if (terminal == QLatin1String("zsh") && cursorLine > 0 && (_lineProperties.at(cursorLine - 1) & LINE_WRAPPED) != 0) {
+                while (cursorLine + cursorLineCorrection > 0 && (_lineProperties.at(cursorLine + cursorLineCorrection - 1) & LINE_WRAPPED) != 0) {
+                    --cursorLineCorrection;
                 }
             }
         }
@@ -476,28 +474,28 @@ void Screen::resizeImage(int new_lines, int new_columns)
         int currentPos = 0;
         while (currentPos < (cursorLine + cursorLineCorrection) && currentPos < _screenLines.count() - 1) {
             // Join wrapped line in current position
-            if ((_lineProperties[currentPos] & LINE_WRAPPED) != 0) {
-                _screenLines[currentPos].append(std::move(_screenLines[currentPos + 1]));
+            if ((_lineProperties.at(currentPos) & LINE_WRAPPED) != 0) {
+                _screenLines[currentPos].append(_screenLines.at(currentPos + 1));
                 _screenLines.remove(currentPos + 1);
                 _lineProperties.remove(currentPos);
-                cursorLine--;
+                --cursorLine;
                 continue;
             }
 
             // Ignore whitespaces at the end of the line
-            int lineSize = _screenLines[currentPos].size();
-            while (lineSize > 0 && QChar(_screenLines[currentPos][lineSize - 1].character).isSpace()) {
-                lineSize--;
+            int lineSize = _screenLines.at(currentPos).size();
+            while (lineSize > 0 && QChar(_screenLines.at(currentPos).at(lineSize - 1).character).isSpace()) {
+                --lineSize;
             }
 
             // If need to move to line below, copy from the current line, to the next one.
-            if (lineSize > new_columns) {
-                auto values = _screenLines[currentPos].mid(new_columns);
+            if (lineSize > new_columns && !(_lineProperties.at(currentPos) & (LINE_DOUBLEHEIGHT_BOTTOM | LINE_DOUBLEHEIGHT_TOP))) {
+                auto values = _screenLines.at(currentPos).mid(new_columns);
                 _screenLines[currentPos].resize(new_columns);
-                _lineProperties.insert(currentPos + 1, _lineProperties[currentPos]);
+                _lineProperties.insert(currentPos + 1, _lineProperties.at(currentPos));
                 _screenLines.insert(currentPos + 1, std::move(values));
                 _lineProperties[currentPos] |= LINE_WRAPPED;
-                cursorLine++;
+                ++cursorLine;
             }
             currentPos += 1;
         }
@@ -506,7 +504,7 @@ void Screen::resizeImage(int new_lines, int new_columns)
     // Check if it need to move from _screenLine to _history
     while (cursorLine > new_lines - 1) {
         fastAddHistLine();
-        cursorLine--;
+        --cursorLine;
     }
 
     if (_enableReflowLines) {
@@ -516,13 +514,13 @@ void Screen::resizeImage(int new_lines, int new_columns)
         while (cursorLine < oldCursorLine && _history->getLines()) {
             int histPos = _history->getLines() - 1;
             int histLineLen = _history->getLineLen(histPos);
-            int isWrapped = _history->isWrappedLine(histPos) ? LINE_WRAPPED : LINE_DEFAULT;
+            LineProperty lineProperty = _history->getLineProperty(histPos);
             histLine.resize(histLineLen);
             _history->getCells(histPos, 0, histLineLen, histLine.data());
             _screenLines.insert(0, histLine);
-            _lineProperties.insert(0, isWrapped);
-            _history->removeCells(histPos);
-            cursorLine++;
+            _lineProperties.insert(0, lineProperty);
+            _history->removeCells();
+            ++cursorLine;
         }
     }
 
@@ -607,21 +605,24 @@ void Screen::copyFromHistory(Character* dest, int startLine, int count) const
 {
     Q_ASSERT(startLine >= 0 && count > 0 && startLine + count <= _history->getLines());
 
-    for (int line = startLine; line < startLine + count; line++) {
+    for (int line = startLine; line < startLine + count; ++line) {
         const int length = qMin(_columns, _history->getLineLen(line));
         const int destLineOffset  = (line - startLine) * _columns;
+        const int lastColumn = (_history->getLineProperty(line) & LINE_DOUBLEWIDTH) ? _columns / 2 : _columns;
 
         _history->getCells(line, 0, length, dest + destLineOffset);
 
         if (length < _columns) {
-            std::fill(&dest[destLineOffset + length], &dest[destLineOffset + _columns], Screen::DefaultChar);
+            const int begin = destLineOffset + length;
+            const int end = destLineOffset + _columns;
+            std::fill(dest + begin, dest + end, Screen::DefaultChar);
         }
 
         // invert selected text
         if (_selBegin != -1) {
-            for (int column = 0; column < _columns; column++) {
+            for (int column = 0; column < lastColumn; ++column) {
                 if (isSelected(column, line)) {
-                    dest[destLineOffset + column].rendition |= RE_SELECTED;
+                    setTextSelectionRendition(dest[destLineOffset + column]);
                 }
             }
         }
@@ -630,26 +631,43 @@ void Screen::copyFromHistory(Character* dest, int startLine, int count) const
 
 void Screen::copyFromScreen(Character* dest , int startLine , int count) const
 {
-    Q_ASSERT(startLine >= 0 && count > 0 && startLine + count <= _lines);
+    const int endLine = startLine + count;
 
-    for (int line = startLine; line < (startLine + count) ; line++) {
-        int srcLineStartIndex  = line * _columns;
-        int destLineStartIndex = (line - startLine) * _columns;
+    Q_ASSERT(startLine >= 0 && count > 0 && endLine <= _lines);
 
-        for (int column = 0; column < _columns; column++) {
-            int srcIndex = srcLineStartIndex + column;
-            int destIndex = destLineStartIndex + column;
+    for (int line = startLine; line < endLine; ++line) {
+        const int srcLineStartIndex  = line * _columns;
+        const int destLineStartIndex = (line - startLine) * _columns;
+        const int lastColumn = (line < _lineProperties.size() && _lineProperties[line] & LINE_DOUBLEWIDTH) ? _columns / 2 : _columns;
 
-            dest[destIndex] = _screenLines[srcIndex / _columns].value(srcIndex % _columns, Screen::DefaultChar);
+        for (int column = 0; column < _columns; ++column) {
+            const int srcIndex = srcLineStartIndex + column;
+            const int destIndex = destLineStartIndex + column;
 
-            // invert selected text
-            if (_selBegin != -1 && isSelected(column, line + _history->getLines())) {
-                dest[destIndex].rendition |= RE_SELECTED;
+            dest[destIndex] = _screenLines.at(srcIndex / _columns).value(srcIndex % _columns, Screen::DefaultChar);
+
+            if (_selBegin != -1 && isSelected(column, line + _history->getLines()) && column < lastColumn) {
+                setTextSelectionRendition(dest[destIndex]);
             }
         }
     }
 }
 
+void Screen::setTextSelectionRendition(Character &ch) const
+{
+    Q_ASSERT(_currentTerminalDisplay);
+
+    auto currentProfile = SessionManager::instance()->sessionProfile(_currentTerminalDisplay->session());
+    const bool isInvert = currentProfile ? currentProfile->property<bool>(Profile::InvertSelectionColors)
+                                         : false;
+
+    if (isInvert) {
+        reverseRendition(ch);
+    } else {
+        ch.rendition |= RE_BLEND_SELECTION_COLORS;
+    }
+}
+
 void Screen::getImage(Character* dest, int size, int startLine, int endLine) const
 {
     Q_ASSERT(startLine >= 0);
@@ -677,12 +695,12 @@ void Screen::getImage(Character* dest, int size, int startLine, int endLine) con
 
     // invert display when in screen mode
     if (getMode(MODE_Screen)) {
-        for (int i = 0; i < mergedLines * _columns; i++) {
+        for (int i = 0; i < mergedLines * _columns; ++i) {
             reverseRendition(dest[i]); // for reverse display
         }
     }
 
-    int visX = qMin(_cuX, _columns - 1);
+    int visX = qMin(_cuX, getScreenLineColumns(_cuY) - 1);
     // mark the character at the current cursor position
     int cursorIndex = loc(visX, _cuY + linesInHistoryBuffer);
     if (getMode(MODE_Cursor) && cursorIndex < _columns * mergedLines) {
@@ -703,24 +721,30 @@ QVector<LineProperty> Screen::getLineProperties(int startLine , int endLine) con
     int index = 0;
 
     // copy properties for _lines in history
-    for (int line = startLine; line < startLine + linesInHistory; line++) {
-        //TODO Support for line properties other than wrapped _lines
-        if (_history->isWrappedLine(line)) {
-            result[index] = static_cast<LineProperty>(result[index] | LINE_WRAPPED);
-        }
-        index++;
+    for (int line = startLine; line < startLine + linesInHistory; ++line) {
+        result[index] = _history->getLineProperty(line);
+        ++index;
     }
 
     // copy properties for _lines in screen buffer
     const int firstScreenLine = startLine + linesInHistory - _history->getLines();
-    for (int line = firstScreenLine; line < firstScreenLine + linesInScreen; line++) {
-        result[index] = _lineProperties[line];
-        index++;
+    for (int line = firstScreenLine; line < firstScreenLine + linesInScreen; ++line) {
+        result[index] = _lineProperties.at(line);
+        ++index;
     }
 
     return result;
 }
 
+int Screen::getScreenLineColumns(const int line) const
+{
+    if (line < _lineProperties.size() && _lineProperties.at(line) & LINE_DOUBLEWIDTH) {
+        return _columns / 2;
+    }
+
+    return _columns;
+}
+
 void Screen::reset()
 {
     // Clear screen, but preserve the current line
@@ -754,7 +778,7 @@ void Screen::backspace()
 {
     _cuX = qMax(0, _cuX - 1);
 
-    if (_screenLines[_cuY].size() < _cuX + 1) {
+    if (_screenLines.at(_cuY).size() < _cuX + 1) {
         _screenLines[_cuY].resize(_cuX + 1);
     }
 }
@@ -765,12 +789,12 @@ void Screen::tab(int n)
     if (n < 1) {
         n = 1;
     }
-    while ((n > 0) && (_cuX < _columns - 1)) {
+    while ((n > 0) && (_cuX < getScreenLineColumns(_cuY) - 1)) {
         cursorRight(1);
-        while ((_cuX < _columns - 1) && !_tabStops[_cuX]) {
+        while ((_cuX < getScreenLineColumns(_cuY) - 1) && !_tabStops.at(_cuX)) {
             cursorRight(1);
         }
-        n--;
+        --n;
     }
 }
 
@@ -782,10 +806,10 @@ void Screen::backtab(int n)
     }
     while ((n > 0) && (_cuX > 0)) {
         cursorLeft(1);
-        while ((_cuX > 0) && !_tabStops[_cuX]) {
+        while ((_cuX > 0) && !_tabStops.at(_cuX)) {
             cursorLeft(1);
         }
-        n--;
+        --n;
     }
 }
 
@@ -810,7 +834,7 @@ void Screen::initTabStops()
     // The 1st tabstop has to be one longer than the other.
     // i.e. the kids start counting from 0 instead of 1.
     // Other programs might behave correctly. Be aware.
-    for (int i = 0; i < _columns; i++) {
+    for (int i = 0; i < _columns; ++i) {
         _tabStops[i] = (i % 8 == 0 && i != 0);
     }
 }
@@ -854,14 +878,14 @@ void Screen::displayCharacter(uint c)
             return;
         }
         // Find previous "real character" to try to combine with
-        int charToCombineWithX = qMin(_cuX, _screenLines[_cuY].length());
+        int charToCombineWithX = qMin(_cuX, _screenLines.at(_cuY).length());
         int charToCombineWithY = _cuY;
         do {
             if (charToCombineWithX > 0) {
-                charToCombineWithX--;
+                --charToCombineWithX;
             } else if (charToCombineWithY > 0) { // Try previous line
-                charToCombineWithY--;
-                charToCombineWithX = _screenLines[charToCombineWithY].length() - 1;
+                --charToCombineWithY;
+                charToCombineWithX = _screenLines.at(charToCombineWithY).length() - 1;
             } else {
                 // Give up
                 return;
@@ -871,7 +895,7 @@ void Screen::displayCharacter(uint c)
             if (charToCombineWithX < 0) {
                 return;
             }
-        } while(!_screenLines[charToCombineWithY][charToCombineWithX].isRealCharacter);
+        } while(!_screenLines.at(charToCombineWithY).at(charToCombineWithX).isRealCharacter);
 
         Character& currentChar = _screenLines[charToCombineWithY][charToCombineWithX];
         if ((currentChar.rendition & RE_EXTENDED_CHAR) == 0) {
@@ -886,28 +910,27 @@ void Screen::displayCharacter(uint c)
             if (((oldChars) != nullptr) && extendedCharLength < 3) {
                 Q_ASSERT(extendedCharLength > 1);
                 Q_ASSERT(extendedCharLength < 65535); // redundant due to above check
-                auto chars = new uint[extendedCharLength + 1];
-                memcpy(chars, oldChars, sizeof(uint) * extendedCharLength);
+                auto chars = std::make_unique<uint[]>(extendedCharLength + 1);
+                std::copy_n(oldChars, extendedCharLength, chars.get());
                 chars[extendedCharLength] = c;
                 auto extChars = [this]() { return usedExtendedChars(); };
-                currentChar.character = ExtendedCharTable::instance.createExtendedChar(chars, extendedCharLength + 1, extChars);
-                delete[] chars;
+                currentChar.character = ExtendedCharTable::instance.createExtendedChar(chars.get(), extendedCharLength + 1, extChars);
             }
         }
         return;
     }
 
-    if (_cuX + w > _columns) {
+    if (_cuX + w > getScreenLineColumns(_cuY)) {
         if (getMode(MODE_Wrap)) {
-            _lineProperties[_cuY] = static_cast<LineProperty>(_lineProperties[_cuY] | LINE_WRAPPED);
+            _lineProperties[_cuY] = static_cast<LineProperty>(_lineProperties.at(_cuY) | LINE_WRAPPED);
             nextLine();
         } else {
-            _cuX = qMax(_columns - w, 0);
+            _cuX = qMax(getScreenLineColumns(_cuY) - w, 0);
         }
     }
 
     // ensure current line vector has enough elements
-    if (_screenLines[_cuY].size() < _cuX + w) {
+    if (_screenLines.at(_cuY).size() < _cuX + w) {
         _screenLines[_cuY].reserve(_columns);
         _screenLines[_cuY].resize(_cuX + w);
     }
@@ -934,9 +957,9 @@ void Screen::displayCharacter(uint c)
     int i = 0;
     const int newCursorX = _cuX + w--;
     while (w != 0) {
-        i++;
+        ++i;
 
-        if (_screenLines[_cuY].size() < _cuX + i + 1) {
+        if (_screenLines.at(_cuY).size() < _cuX + i + 1) {
             _screenLines[_cuY].resize(_cuX + i + 1);
         }
 
@@ -947,7 +970,7 @@ void Screen::displayCharacter(uint c)
         ch.rendition = _effectiveRendition;
         ch.isRealCharacter = false;
 
-        w--;
+        --w;
     }
     _cuX = newCursorX;
     _escapeSequenceUrlExtractor->appendUrlText(QChar(c));
@@ -1073,7 +1096,7 @@ int Screen::getCursorY() const
     return _cuY;
 }
 
-void Screen::clearImage(int loca, int loce, char c)
+void Screen::clearImage(int loca, int loce, char c, bool resetLineRendition)
 {
     const int scr_TL = loc(0, _history->getLines());
     //FIXME: check positions
@@ -1092,8 +1115,8 @@ void Screen::clearImage(int loca, int loce, char c)
     //default character, the affected _lines can simply be shrunk.
     const bool isDefaultCh = (clearCh == Screen::DefaultChar);
 
-    for (int y = topLine; y <= bottomLine; y++) {
-        _lineProperties[y] = 0;
+    for (int y = topLine; y <= bottomLine; ++y) {
+        _lineProperties[y] &= ~LINE_WRAPPED;
 
         const int endCol = (y == bottomLine) ? loce % _columns : _columns - 1;
         const int startCol = (y == topLine) ? loca % _columns : 0;
@@ -1111,6 +1134,10 @@ void Screen::clearImage(int loca, int loce, char c)
                 std::fill(line.begin() + startCol, line.begin() + (endCol + 1), clearCh);
             }
         }
+
+        if (resetLineRendition && startCol == 0 && endCol == _columns - 1) {
+                _lineProperties[y] &= ~(LINE_DOUBLEWIDTH | LINE_DOUBLEHEIGHT_TOP | LINE_DOUBLEHEIGHT_BOTTOM);
+        }
     }
 }
 
@@ -1128,14 +1155,14 @@ void Screen::moveImage(int dest, int sourceBegin, int sourceEnd)
     const int destY = dest / _columns;
     const int srcY = sourceBegin / _columns;
     if (dest < sourceBegin) {
-        for (int i = 0; i <= lines; i++) {
-            _screenLines[destY + i] = _screenLines[srcY + i];
-            _lineProperties[destY + i] = _lineProperties[srcY + i];
+        for (int i = 0; i <= lines; ++i) {
+            _screenLines[destY + i] = _screenLines.at(srcY + i);
+            _lineProperties[destY + i] = _lineProperties.at(srcY + i);
         }
     } else {
-        for (int i = lines; i >= 0; i--) {
+        for (int i = lines; i >= 0; --i) {
             _screenLines[destY + i ] = std::move(_screenLines[srcY + i]);
-            _lineProperties[destY + i] = _lineProperties[srcY + i];
+            _lineProperties[destY + i] = _lineProperties.at(srcY + i);
         }
     }
 
@@ -1211,17 +1238,17 @@ void Screen::helpAlign()
 
 void Screen::clearToEndOfLine()
 {
-    clearImage(loc(_cuX, _cuY), loc(_columns - 1, _cuY), ' ');
+    clearImage(loc(_cuX, _cuY), loc(_columns - 1, _cuY), ' ', false);
 }
 
 void Screen::clearToBeginOfLine()
 {
-    clearImage(loc(0, _cuY), loc(_cuX, _cuY), ' ');
+    clearImage(loc(0, _cuY), loc(_cuX, _cuY), ' ', false);
 }
 
 void Screen::clearEntireLine()
 {
-    clearImage(loc(0, _cuY), loc(_columns - 1, _cuY), ' ');
+    clearImage(loc(0, _cuY), loc(_columns - 1, _cuY), ' ', false);
 }
 
 void Screen::setRendition(RenditionFlags rendition)
@@ -1240,7 +1267,7 @@ void Screen::setDefaultRendition()
 {
     setForeColor(COLOR_SPACE_DEFAULT, DEFAULT_FORE_COLOR);
     setBackColor(COLOR_SPACE_DEFAULT, DEFAULT_BACK_COLOR);
-    _currentRendition   = DEFAULT_RENDITION;
+    _currentRendition = DEFAULT_RENDITION;
     updateEffectiveRendition();
 }
 
@@ -1303,7 +1330,7 @@ void Screen::setSelectionStart(const int x, const int y, const bool blockSelecti
     _selBegin = loc(x, y);
     /* FIXME, HACK to correct for x too far to the right... */
     if (x == _columns) {
-        _selBegin--;
+        --_selBegin;
     }
 
     _selBottomRight = _selBegin;
@@ -1325,7 +1352,7 @@ void Screen::setSelectionEnd(const int x, const int y)
     } else {
         /* FIXME, HACK to correct for x too far to the right... */
         if (x == _columns) {
-            endPos--;
+            --endPos;
         }
 
         _selTopLeft = _selBegin;
@@ -1404,7 +1431,7 @@ void Screen::writeToStream(TerminalCharacterDecoder* decoder,
 
     Q_ASSERT(top >= 0 && left >= 0 && bottom >= 0 && right >= 0);
 
-    for (int y = top; y <= bottom; y++) {
+    for (int y = top; y <= bottom; ++y) {
         int start = 0;
         if (y == top || _blockSelectionMode) {
             start = left;
@@ -1421,6 +1448,7 @@ void Screen::writeToStream(TerminalCharacterDecoder* decoder,
                                       count,
                                       decoder,
                                       appendNewLine,
+                                      _blockSelectionMode,
                                       options);
 
         // if the selection goes beyond the end of the last line then
@@ -1470,6 +1498,7 @@ int Screen::copyLineToStream(int line ,
                              int count,
                              TerminalCharacterDecoder* decoder,
                              bool appendNewLine,
+                             bool isBlockSelectionMode,
                              const DecodingOptions options) const
 {
     const int lineLength = getLineLength(line);
@@ -1516,10 +1545,10 @@ int Screen::copyLineToStream(int line ,
         screenLine = qMin(screenLine, _screenLinesSize);
 
         auto* data = _screenLines[screenLine].data();
-        int length = _screenLines[screenLine].count();
+        int length = _screenLines.at(screenLine).count();
 
         // Don't remove end spaces in lines that wrap
-        if (options.testFlag(TrimTrailingWhitespace) && ((_lineProperties[screenLine] & LINE_WRAPPED) == 0))
+        if (options.testFlag(TrimTrailingWhitespace) && ((_lineProperties.at(screenLine) & LINE_WRAPPED) == 0))
         {
             // ignore trailing white space at the end of the line
             while (length > 0 && QChar(data[length - 1].character).isSpace()) {
@@ -1541,20 +1570,19 @@ int Screen::copyLineToStream(int line ,
     }
 
     if (appendNewLine) {
-        if ((currentLineProperties & LINE_WRAPPED) != 0) {
-            // do nothing extra when this line is wrapped.
-        } else {
-            // When users ask not to preserve the linebreaks, they usually mean:
-            // `treat LINEBREAK as SPACE, thus joining multiple _lines into
-            // single line in the same way as 'J' does in VIM.`
+        // When users ask not to preserve the linebreaks, they usually mean:
+        // `treat LINEBREAK as SPACE, thus joining multiple _lines into
+        // single line in the same way as 'J' does in VIM.`
+        const bool isLineWrapped = (currentLineProperties & LINE_WRAPPED) != 0;
+        if (isBlockSelectionMode || !isLineWrapped) {
             characterBuffer[count] = options.testFlag(PreserveLineBreaks) ? Character('\n') : Character(' ');
-            count++;
+            ++count;
         }
     }
 
     if ((options & TrimLeadingWhitespace) != 0U) {
         int spacesCount = 0;
-        for (spacesCount = 0; spacesCount < count; spacesCount++) {
+        for (spacesCount = 0; spacesCount < count; ++spacesCount) {
             if (QChar::category(characterBuffer[spacesCount].character) != QChar::Category::Separator_Space) {
                 break;
             }
@@ -1584,8 +1612,8 @@ void Screen::writeLinesToStream(TerminalCharacterDecoder* decoder, int fromLine,
 void Screen::fastAddHistLine()
 {
     const bool removeLine = _history->getLines() == _history->getMaxLines();
-    _history->addCellsVector(_screenLines[0]);
-    _history->addLine((_lineProperties[0] & LINE_WRAPPED) != 0);
+    _history->addCellsVector(_screenLines.at(0));
+    _history->addLine(_lineProperties.at(0));
 
     // If _history size > max history size it will drop a line from _history.
     // We need to verify if we need to remove a URL.
@@ -1593,8 +1621,8 @@ void Screen::fastAddHistLine()
         _escapeSequenceUrlExtractor->historyLinesRemoved(1);
     }
 
-    _screenLines.pop_front();
-    _lineProperties.remove(0);
+    _screenLines.removeFirst();
+    _lineProperties.removeFirst();
 }
 
 void Screen::addHistLine()
@@ -1605,15 +1633,15 @@ void Screen::addHistLine()
     int newHistLines = _history->getLines();
 
     if (hasScroll()) {
-        _history->addCellsVector(_screenLines[0]);
-        _history->addLine((_lineProperties[0] & LINE_WRAPPED) != 0);
+        _history->addCellsVector(_screenLines.at(0));
+        _history->addLine(_lineProperties.at(0));
 
         newHistLines = _history->getLines();
 
         // If the history is full, increment the count
         // of dropped _lines
         if (newHistLines == oldHistLines) {
-            _droppedLines++;
+            ++_droppedLines;
 
             // We removed a line, we need to verify if we need to remove a URL.
             _escapeSequenceUrlExtractor->historyLinesRemoved(1);
@@ -1663,16 +1691,17 @@ int Screen::getHistLines() const
     return _history->getLines();
 }
 
-void Screen::setScroll(const HistoryType& t , bool copyPreviousScroll)
+void Screen::setScroll(const HistoryType &t, bool copyPreviousScroll)
 {
     clearSelection();
 
     if (copyPreviousScroll) {
-        _history = t.scroll(_history);
+        t.scroll(_history);
     } else {
-        HistoryScroll* oldScroll = _history;
-        _history = t.scroll(nullptr);
-        delete oldScroll;
+        // As 't' can be '_history' pointer, move it to a temporary smart pointer
+        // making _history = nullptr
+        auto oldHistory = std::move(_history);
+        t.scroll(_history);
     }
 }
 
@@ -1681,7 +1710,7 @@ bool Screen::hasScroll() const
     return _history->hasScroll();
 }
 
-const HistoryType& Screen::getScroll() const
+const HistoryType &Screen::getScroll() const
 {
     return _history->getType();
 }
@@ -1689,9 +1718,9 @@ const HistoryType& Screen::getScroll() const
 void Screen::setLineProperty(LineProperty property , bool enable)
 {
     if (enable) {
-        _lineProperties[_cuY] = static_cast<LineProperty>(_lineProperties[_cuY] | property);
+        _lineProperties[_cuY] = static_cast<LineProperty>(_lineProperties.at(_cuY) | property);
     } else {
-        _lineProperties[_cuY] = static_cast<LineProperty>(_lineProperties[_cuY] & ~property);
+        _lineProperties[_cuY] = static_cast<LineProperty>(_lineProperties.at(_cuY) & ~property);
     }
 }
 void Screen::fillWithDefaultChar(Character* dest, int count)
@@ -1701,5 +1730,5 @@ void Screen::fillWithDefaultChar(Character* dest, int count)
 
 Konsole::EscapeSequenceUrlExtractor * Konsole::Screen::urlExtractor() const
 {
-    return _escapeSequenceUrlExtractor;
+    return _escapeSequenceUrlExtractor.get();
 }
