@@ -20,10 +20,11 @@ typedef unsigned char LineProperty;
 
 typedef quint16 RenditionFlags;
 
-const int LINE_DEFAULT      = 0;
-const int LINE_WRAPPED      = (1 << 0);
-const int LINE_DOUBLEWIDTH  = (1 << 1);
-const int LINE_DOUBLEHEIGHT = (1 << 2);
+const int LINE_DEFAULT             = 0;
+const int LINE_WRAPPED             = (1 << 0);
+const int LINE_DOUBLEWIDTH         = (1 << 1);
+const int LINE_DOUBLEHEIGHT_TOP    = (1 << 2);
+const int LINE_DOUBLEHEIGHT_BOTTOM = (1 << 3);
 
 const RenditionFlags DEFAULT_RENDITION = 0;
 const RenditionFlags RE_BOLD           = (1 << 0);
@@ -37,7 +38,7 @@ const RenditionFlags RE_FAINT          = (1 << 7);
 const RenditionFlags RE_STRIKEOUT      = (1 << 8);
 const RenditionFlags RE_CONCEAL        = (1 << 9);
 const RenditionFlags RE_OVERLINE       = (1 << 10);
-const RenditionFlags RE_SELECTED       = (1 << 11);
+const RenditionFlags RE_BLEND_SELECTION_COLORS = (1 << 11);
 
 /**
  * A single character in the terminal which consists of a unicode character
