@@ -23,10 +23,7 @@ ColorSchemeWallpaper::ColorSchemeWallpaper(const QString &path) :
 {
 }
 
-ColorSchemeWallpaper::~ColorSchemeWallpaper()
-{
-    delete _picture;
-}
+ColorSchemeWallpaper::~ColorSchemeWallpaper() = default;
 
 void ColorSchemeWallpaper::load()
 {
@@ -36,7 +33,7 @@ void ColorSchemeWallpaper::load()
 
     // Create and load original pixmap
     if (_picture == nullptr) {
-        _picture = new QPixmap();
+        _picture = std::make_unique<QPixmap>();
     }
 
     if (_picture->isNull()) {
