@@ -118,7 +118,7 @@ Session::Session(QObject* parent) :
 
     connect(_emulation, &Konsole::Emulation::sessionAttributeChanged, this, &Konsole::Session::setSessionAttribute);
     connect(_emulation, &Konsole::Emulation::bell, this, [this]() {
-        Q_EMIT bellRequest(i18n("Bell in session '%1'", _nameTitle));
+        Q_EMIT bellRequest(i18n("Bell in '%1' (Session '%2')", _displayTitle, _nameTitle));
         this->setPendingNotification(Notification::Bell);
     });
     connect(_emulation, &Konsole::Emulation::zmodemDownloadDetected, this, &Konsole::Session::fireZModemDownloadDetected);
@@ -339,6 +339,8 @@ void Session::addView(TerminalDisplay* widget)
 
     widget->setScreenWindow(_emulation->createWindow());
 
+    _emulation->setCurrentTerminalDisplay(widget);
+
     //connect view signals and slots
     connect(widget, &Konsole::TerminalDisplay::changedContentSizeSignal, this, &Konsole::Session::onViewSizeChange);
 
@@ -472,15 +474,17 @@ void Session::run()
         }
     }
 
-    // if none of the choices are available, print a warning
+    // if nothing could be found (not even the fallbacks), print a warning and do not run
     if (exec.isEmpty()) {
         terminalWarning(i18n("Could not find an interactive shell to start."));
         return;
     }
 
-    // if a program was specified via setProgram(), but it couldn't be found, print a warning
+    // if a program was specified via setProgram(), but it couldn't be found (but a fallback was), print a warning
     if (exec != checkProgram(_program)) {
         terminalWarning(i18n("Could not find '%1', starting '%2' instead.  Please check your profile settings.", _program, exec));
+    } else if ( exec != checkProgram(exec)) {
+        terminalWarning(i18n("Could not find '%1', starting '%2' instead.  Please check your profile settings.", exec, checkProgram(exec)));
     }
 
     // if no arguments are specified, fall back to program name
@@ -657,7 +661,7 @@ void Session::silenceTimerDone()
     }
 
     KNotification::event(hasFocus() ? QStringLiteral("Silence") : QStringLiteral("SilenceHidden"),
-            i18n("Silence in session '%1'", _nameTitle), QPixmap(),
+            i18n("Silence in '%1' (Session '%2')", _displayTitle, _nameTitle), QPixmap(),
             QApplication::activeWindow(),
             KNotification::CloseWhenWidgetActivated);
     setPendingNotification(Notification::Silence);
@@ -1743,7 +1747,7 @@ void Session::handleActivity()
 
     if (_monitorActivity && !_notifiedActivity) {
         KNotification::event(hasFocus() ? QStringLiteral("Activity") : QStringLiteral("ActivityHidden"),
-                             i18n("Activity in session '%1'", _nameTitle), QPixmap(),
+                             i18n("Activity in '%1' (Session '%2')", _displayTitle, _nameTitle), QPixmap(),
                              QApplication::activeWindow(),
                              KNotification::CloseWhenWidgetActivated);
 
